package com.txtnation.demo;

import java.util.List;
import java.util.Random;

import com.txtnation.android.BillingManager;
import com.txtnation.android.Environment;
import com.txtnation.android.PaymentListener;
import com.txtnation.android.products.Product;

import android.app.Activity;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

/**
 * This is a Sample Payment Class that shows how you can integrate TAL with your
 * application.
 */
public class SampleAppPaymentActivity extends Activity implements
		PaymentListener {
	private Button paymentButton;
	private BillingManager billingManager;
	public long accountId;
	private String endUserId, clientAppVersion;
	private boolean isOneStepBilling;
	private String message;
	private Product selectedProduct;

	// We need a handler for call-backs to the UI thread.
	// the BillingManager class will not work without a handler.
	private final Handler mHandler = new Handler();

	Runnable toastRunnable = new Runnable() {
		public void run() {
			Toast
					.makeText(SampleAppPaymentActivity.this, message, Toast.LENGTH_SHORT)
					.show();
		}
	};

	Runnable manualPurchaseRunnable = new Runnable() {
		public void run() {

			billingManager.purchaseProduct(selectedProduct);

		}
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		paymentButton = (Button) findViewById(R.id.paymentButton);
		paymentButton.setOnClickListener(new PaymentOnClickListener());

		// instantiate the BillingManager with the current context.
		billingManager = new BillingManager(SampleAppPaymentActivity.this, mHandler);

		billingManager.setEnvironment(Environment.PRODUCTION);
		// billingManager.setEnvironment(Environment.LOCAL);

	}

	class PaymentOnClickListener implements OnClickListener {

		public void onClick(View v) {

			try {
				/*
				 * Unique account ID generated by txtNation for the client application
				 * or Game integrating with TAL. Using this magic ID txtNation retrieves
				 * all Client specific details from txtNation Servers, details such as
				 * Prices available in each country, subscription information,
				 * regulations, customer service info billing information etc.
				 */

				accountId = 9599590; // ACCOUNT ID HERE

				/*
				 * A unique endUserId (could be the login ID of the customer using the
				 * game) or otherwise any unique transaction ID that identifies the user
				 * paying for your service. The value you set here will be passed to
				 * your servers on all communications
				 */
				endUserId = "testUser1";

				// Enable ONE STEP BILLING (if true displays two billing screens to the
				// user, otherwise one billing screen only )
				isOneStepBilling = true;

				// gets application version from the Manifest file
				// android:versionName=""
				clientAppVersion = getPackageManager().getPackageInfo(getPackageName(),
						0).versionName;

			} catch (NameNotFoundException e) {
				// TODO Auto-generated catch block
			}

			// billingManager.setTestNumber("+447947755628");
			// billingManager.setTestNumber("5558");

			// Register this class 'SampleAppPaymentActivity' as payment
			// listener

			billingManager.registerPaymentListener(SampleAppPaymentActivity.this);

			// pass the client app specific information to BillingManage's
			// beginPayment() method, the method should return true if payment
			// has began.
			boolean isPaymentBegan = billingManager.beginPayment(accountId,
					endUserId, clientAppVersion, isOneStepBilling);
		}

	}

	public void onProductsReceived(final List<Product> products) {
		/*
		 * if isOneStepBilling == false, developers have to manually call
		 * billingManager.purchaseProduct(product) method, with one of the products
		 * returned in the parameter products array .Developers may want to display
		 * a dialog of their own to list the products.
		 */

		if (isOneStepBilling == false) {
			// Fake a user randomly selecting a product from the Product
			// Selection
			// Screen as we don't have one in this Payment activity
			Random ran = new Random();

			StringBuilder builder = new StringBuilder();
			for (Product p : products) {
				builder.append("Product ID ");
				builder.append(p.getProductId());
				builder.append("\n");
				builder.append("Product Name ");
				builder.append(p.getProductName());
				builder.append("\n");
				// This is the price point to charge the user the first time
				// they subscribe or if non-subscription the only price to
				// charge the user
				builder.append("Initial Price Point ");
				builder.append(p.getInitialPricePoint());
				builder.append("\n");
				builder.append("Currency ");
				builder.append(p.getCurrency());
				builder.append("\n");
				builder.append("....................");
				builder.append("\n");
			}

			this.message = "A list of products has been received. Total products "
					+ products.size() + " products = " + builder.toString();
			mHandler.post(toastRunnable);

			int i = ran.nextInt(products.size());
			selectedProduct = products.get(i);
			if (selectedProduct != null) {
				// use handler so that this is run on the User Interface thread.
				mHandler.post(manualPurchaseRunnable);
			}
		} else {
			/*
			 * if isOneStepBilling is true then developer doesn't have to do anything.
			 * The billing library will take care this for you. However, regardless,
			 * the value of isOneStepBilling, this method will be called.
			 */
		}

	}

	public void onProductSelected(Product product) {
		// this.message = "A Product has been selected. Product Id is "
		// + product.getProductId() + " and price is " +
		// product.getInitialPricePoint();
		// mHandler.post(toastRunnable);

	}

	public void onCancelled() {
		this.message = "Payment has been cancelled.";
		mHandler.post(toastRunnable);
	}

	public void onFailure(String failureDetails) {
		this.message = failureDetails;
		mHandler.post(toastRunnable);

	}

	public void onSuccess() {
		this.message = "Payment has been successful....";
		mHandler.post(toastRunnable);

		/*
		 * Client application should credit or topup the user's account here if the
		 * user account crediting is successful call
		 * billingManager.sendAcknowledgementSuccess with a transactionId that
		 * exists in your system if any
		 */
		String clientTransactionId = "EXAMPLE3393939";
		boolean userAccountToppedUp = true;

		if (userAccountToppedUp) {
			billingManager.sendAcknowledgementSuccess(clientTransactionId);
		} else {
			billingManager.sendAcknowledgementFailure(clientTransactionId,
					"Reason why your system could not top up users account");
		}

	}

	public void onPaymentStateChanged(String paymentState) {
		/*
		 * this.message = "Payment status is " + paymentState;
		 * mHandler.post(toastRunnable);
		 */

	}

}
